//
//  ViewController.m
//  Rotation
//
//  Created by Clemens Wagner on 20.06.15.
//  Copyright (c) 2015 Clemens Wagner. All rights reserved.
//

#import "ViewController.h"
#import "UIView+Geometry.h"
#import "PictureView.h"
#import "FrameView.h"

#import <QuartzCore/QuartzCore.h>

const CGFloat kStartAngle = M_PI / 9.0;

@interface ViewController ()

@property (weak, nonatomic) IBOutlet PictureView *pictureView;
@property (weak, nonatomic) IBOutlet FrameView *frameView;
@property (nonatomic) CGPoint startPoint;
@property (nonatomic) CGFloat offset;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *pictureCenterXConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *pictureCenterYConstraint;
@property (weak, nonatomic) IBOutlet UIButton *resetButton;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.frameView.transform = CGAffineTransformMakeRotation(kStartAngle);
    [self reset:nil];
}

- (IBAction)reset:(id)inSender {
    self.frameView.transform = CGAffineTransformMakeRotation(kStartAngle);
    self.pictureView.transform = CGAffineTransformMakeRotation(0.0);
    self.pictureCenterXConstraint.constant = 0.0;
    self.pictureCenterYConstraint.constant = 0.0;
    self.resetButton.alpha = 0.0;
    self.offset = 0.0;
}

- (IBAction)rotatePicture:(UIPanGestureRecognizer *)inGestureRecognizer {
    UIView *theView = self.view;
    CGPoint thePoint = [inGestureRecognizer locationInView:self.view];
    CGFloat theAngle = [theView angleWithPoint:self.startPoint] - [theView angleWithPoint:thePoint];
    
    switch (inGestureRecognizer.state) {
        case UIGestureRecognizerStateBegan:
            self.startPoint = thePoint;
            break;
        case UIGestureRecognizerStateChanged:
            self.frameView.transform = CGAffineTransformMakeRotation(kStartAngle - theAngle);
            self.pictureView.transform = [self makeTransformWithAngle:theAngle];
            break;
        case UIGestureRecognizerStateEnded:
            if(fabs(kStartAngle - theAngle) < 0.1) {
                theAngle = kStartAngle;
                [self performSelector:@selector(animateShip) withObject:nil afterDelay:0.0];
            }
            else if(fabs(theAngle) < 0.1) {
                theAngle = 0.0;
            }
            self.frameView.transform = CGAffineTransformMakeRotation(kStartAngle - theAngle);
            self.pictureView.transform = [self makeTransformWithAngle:theAngle];
            break;
        default:
            break;
    }
}

- (CGAffineTransform)makeTransformWithAngle:(CGFloat)inAngle {
    CGAffineTransform theTransform = CGAffineTransformMakeTranslation(cos(inAngle) * self.offset,
                                                                      sin(inAngle) * self.offset);
    
    return CGAffineTransformRotate(theTransform, inAngle);
}

- (void)animateShip {
    self.offset = CGRectGetWidth(self.view.bounds) / 2.0;
    [self animateShipByConstraints];
}

- (void)animateShipByConstraints {
    self.pictureCenterXConstraint.constant = -cos(kStartAngle) * self.offset;
    self.pictureCenterYConstraint.constant = -sin(kStartAngle) * self.offset;
    [UIView animateWithDuration:2.5 delay:0.5 options:UIViewAnimationOptionCurveEaseIn animations:^{
        [self.pictureView.superview layoutIfNeeded];
    } completion:^(BOOL inFinished) {
        [UIView animateWithDuration:0.5 animations:^{
            self.resetButton.alpha = 1.0;
        }];
    }];
}

- (void)animateShipByTransform {
    [UIView animateWithDuration:2.5 delay:0.5 options:UIViewAnimationOptionCurveEaseIn animations:^{
        self.pictureView.transform = [self makeTransformWithAngle:kStartAngle];
    } completion:^(BOOL inFinished) {
        [UIView animateWithDuration:0.5 animations:^{
            self.resetButton.alpha = 1.0;
        }];
    }];
}

- (void)animateShipByLayer {
    CABasicAnimation *theAnimation = [CABasicAnimation animation];
    
    [CATransaction begin];
    [CATransaction setAnimationDuration:2.5];
    [CATransaction setCompletionBlock:^{
        [UIView animateWithDuration:0.5 animations:^{
            self.resetButton.alpha = 1.0;
        }];
    }];
    self.pictureView.transform = [self makeTransformWithAngle:kStartAngle];
    [self.pictureView.layer addAnimation:theAnimation forKey:@"transform"];
    [CATransaction commit];
}

@end
